-- Example project for "XK2F3 Rev.C" FPGA kit
-- www.PLDkit.com
-- firmware version: 1.0
-- 10.04.2017

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity top is

Port (J5,J6,J7,J8			: in std_logic_vector(7 downto 0);
		nc						: in std_logic_vector(16 downto 0);
		led					: out std_logic_vector(3 downto 0);
		btn					: in std_logic_vector(1 downto 0);
		spkr,tx				: out std_logic;
		sda,scl   	    	: inout std_logic;
		rx,clk,INITB		: in std_logic);
end top;
---------------------------------------------
architecture Behavioral of top is
	
	component uart_tx
	port (
    i_clk       : in  std_logic;
    i_tx_dv     : in  std_logic;
    i_tx_byte   : in  std_logic_vector(7 downto 0);
    o_tx_active : out std_logic;
    o_tx_serial : out std_logic;
    o_tx_done   : out std_logic
    );
	END component;

	component uart_rx
	port (
    i_clk       : in  std_logic;
    i_rx_serial : in  std_logic;
    o_rx_dv     : out std_logic;
    o_rx_byte   : out std_logic_vector(7 downto 0)
    );
	END component;

	COMPONENT i2c_master
	PORT(
		clk 			: IN std_logic;
		reset_n 		: IN std_logic;
		ena 			: IN std_logic;
		addr 			: IN std_logic_vector(6 downto 0);
		rw 			: IN std_logic;
		data_wr 		: IN std_logic_vector(7 downto 0);    
		sda 			: INOUT std_logic;
		scl 			: INOUT std_logic;      
		busy 			: OUT std_logic;
		data_rd 		: OUT std_logic_vector(7 downto 0);
		ack_error	: OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT debounce
	PORT(
		clk : IN std_logic;
		button : IN std_logic;          
		result : OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT DBounce
	PORT(
		clk : IN std_logic;
		nreset : IN std_logic;
		button_in : IN std_logic;          
		DB_out : OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT music
	PORT(
		clk : IN std_logic;          
		speaker : OUT std_logic
		);
	END COMPONENT;
---------------------------------------------
signal i_o					:STD_LOGIC_VECTOR(50 downto 0);
signal timer				:STD_LOGIC_VECTOR(35 downto 0);
signal count_d			 	:STD_LOGIC_VECTOR(24 DOWNTO 0);
signal count_pwr  		:STD_LOGIC_VECTOR(26 DOWNTO 0);
signal rx_data,i2c_data_rd,i2c_data_wr,o_rx_data,i2c_data_write :STD_LOGIC_VECTOR(7 DOWNTO 0);
signal io_sum				:STD_LOGIC_VECTOR(5 DOWNTO 0);
signal mode					:STD_LOGIC_VECTOR(2 downto 0);
signal led0,led1,led2,led3	:STD_LOGIC_VECTOR(3 downto 0);
signal sound1,sound2,rx_dv,s_1,s_2,s_3,tx_dv,ena_i2c_write,power_up,reset,tx_dv_en,lock_rx,prog_mode,io_btn,
io_test_mode,music_mode,five_sec,ena_i2c,ena_i2c_read,stop_iic,i2c_busy,rw_i2c,
rst_i2c,count_start,start_iic,rx_serial,CE,ten_min,rx_data_switch,ok :STD_LOGIC;
---------------------------------------------
begin

	uarttx: uart_tx
	port map(	i_clk			=> clk,
					i_tx_dv		=> tx_dv,
					i_tx_byte	=> i2c_data_rd,
					o_tx_active => open,
					o_tx_done 	=> open,
					o_tx_serial	=> tx);
					
	uartrx: uart_rx
	port map(	i_clk			=> clk,
					i_rx_serial => rx_serial,
					o_rx_dv 		=> rx_dv,
					o_rx_byte	=> o_rx_data);

	Inst_i2c_master: i2c_master PORT MAP(
		clk => clk,
		reset_n => rst_i2c,
		ena => ena_i2c,
		addr => "1010000", -- AT24C1024 EEPROM ADDRESS (set "1010001" for the "second half")
		rw => rw_i2c,
		data_wr => i2c_data_write,
		busy => i2c_busy,
		data_rd => i2c_data_rd,
		ack_error => open,
		sda => sda,
		scl => scl
	);

	Inst_debounce: debounce PORT MAP(
		clk => clk,
		button => btn(0),
		result => reset
	);
	
	Inst_DBounce: DBounce PORT MAP(
		clk => clk,
		nreset => reset,
		button_in => btn(1),
		DB_out => io_btn
	);
	
	Inst_music: music PORT MAP(
		clk => clk,
		speaker => sound2
	);

---------------------------------------------------------
	
	process(clk,reset)
	begin
	  if (reset='0') then
    count_pwr <= (others => '0');
  elsif (clk'event and clk = '1') then
      if (CE='0') then
			count_pwr <= count_pwr + '1';
		end if;
		end if;
	end process;
	
CE <= '1' when count_pwr= X"7735941" else '0';
power_up <= '1' when (count_pwr= X"4FF4C" or count_pwr= X"7735940") else '0';
rx_data_switch  <= power_up or five_sec or ten_min or not reset;

process(rx_data_switch,rx_dv,o_rx_data)
begin
  if (rx_data_switch = '1') then
      rx_data <= "00000000";
  elsif (rx_dv = '1') then
		rx_data <= o_rx_data;
  end if;
end process;

tx_dv <= not i2c_busy and tx_dv_en;
	
s_1 <= '1' when o_rx_data= "00110001" and rx_dv='1' else '0';
s_2 <= '1' when o_rx_data= "00110010" and rx_dv='1'  else '0';
s_3 <= '1' when o_rx_data= "00110011" and rx_dv='1'  else '0';

	process(clk,rx_dv,reset)
	begin
	  if ((rx_dv='1') or (reset='0')) then
    timer <= (others => '0');
  elsif (clk'event and clk = '1') then 
			timer <= timer + '1';
		end if;
	end process;

five_sec <= '1' when timer= X"12A05F20"   and music_mode = '1' else '0';
ten_min  <= '1' when timer= X"8BB2C9700" and io_test_mode = '1' else '0';

process(s_1,five_sec,power_up,reset)
begin
  if ((five_sec = '1') or (power_up = '1') or (reset='0')) then
      music_mode <= '0';
  elsif (s_1 = '1') then
		music_mode <= '1';
  end if;
end process;

process(s_2,power_up,reset)
begin
  if ((power_up = '1') or (reset='0')) then
      prog_mode <= '0';
  elsif (s_2 = '1') then
		prog_mode <= '1';
  end if;
end process;

process(s_3,ten_min,power_up,reset)
begin
  if ((ten_min = '1') or (power_up = '1') or (reset='0')) then
      io_test_mode <= '0';
  elsif (s_3 = '1') then
		io_test_mode <= '1';
  end if;
end process;

mode(0) <= music_mode;
mode(1) <= prog_mode;
mode(2) <= io_test_mode;

process(mode,led1,led2,led3,led0)
begin
	   case mode is
		when "001" => led<=led1;
      when "010" => led<=led2;
		when "100" => led<=led3;
      when others => led<=led0;
   end case;
end process;

lock_rx <= prog_mode or io_test_mode or music_mode or not ce;
rx_serial <= RX when lock_rx = '0' else '1';

i2c_data_write <= "00000000" when (count_pwr > X"3BF79" and count_pwr < X"7723E0C") else i2c_data_wr;

process(rx_data)
begin
	   case rx_data is
		when "00000000" => i2c_data_wr <= "00000111"; --MENU
      when "00110001" => i2c_data_wr <= "00001110"; --MUSIC
		when "00110010" => i2c_data_wr <= "00010101"; --PROG
		when "00110011" => i2c_data_wr <= "00011100"; --IO
      when others => i2c_data_wr <= "00000111"; --MENU
   end case;
end process;

start_iic <= rx_dv or power_up or five_sec or ten_min;

process(start_iic,stop_iic,reset)
begin
  if ((stop_iic = '1') or (reset='0')) then
      count_start <= '0';
  elsif (start_iic = '1') then
		count_start <= '1';
  end if;
end process;

	process(clk,count_start,reset)
	begin
	  if ((count_start='0') or (reset='0')) then
    count_d <= (others => '0');
  elsif (clk'event and clk = '1') then 
			count_d <= count_d + '1';
		end if;
	end process;

stop_iic <= '1' when count_d > X"9FFF79" and i2c_busy='0' else '0';
ena_i2c_write <= '1' when count_d > X"136" and count_d < X"3A98" and count_start='1' else '0';
rw_i2c <= '0' when count_d < X"4E20" else '1';
ena_i2c_read <= '1' when count_d > X"61A8" else '0';
ena_i2c <= ena_i2c_write or ena_i2c_read;
tx_dv_en <= '1' when count_d > X"77A1" else '0';
rst_i2c <= reset;

process(timer)
begin
   case timer(25 downto 23) is 
      when "000" => led0 <= "0010";
		when "001" => led0 <= "0010";
		when "010" => led0 <= "0100";
		when "011" => led0 <= "1000";
		when "100" => led0 <= "0001";
		when "101" => led0 <= "0001";
		when "110" => led0 <= "1000";
		when "111" => led0 <= "0100";
      when others => led0 <= "0000";
   end case;
end process;

LED1(0) <= timer(23);
LED1(1) <= timer(23);
LED1(2) <= timer(23);
LED1(3) <= timer(23);

LED2(0) <= SCL;
LED2(1) <= SCL;
LED2(2) <= SDA;
LED2(3) <= SDA;

i_o <= (J5&J6&J7&J8&nc)&(not io_btn)&(not INITB);

io_sum <=
 ("00000"&i_o(	0	))
+("00000"&i_o(	1	))
+("00000"&i_o(	2	))
+("00000"&i_o(	3	))
+("00000"&i_o(	4	))
+("00000"&i_o(	5	))
+("00000"&i_o(	6	))
+("00000"&i_o(	7	))
+("00000"&i_o(	8	))
+("00000"&i_o(	9	))
+("00000"&i_o(	10	))
+("00000"&i_o(	11	))
+("00000"&i_o(	12	))
+("00000"&i_o(	13	))
+("00000"&i_o(	14	))
+("00000"&i_o(	15	))
+("00000"&i_o(	16	))
+("00000"&i_o(	17	))
+("00000"&i_o(	18	))
+("00000"&i_o(	19	))
+("00000"&i_o(	20	))
+("00000"&i_o(	21	))
+("00000"&i_o(	22	))
+("00000"&i_o(	23	))
+("00000"&i_o(	24	))
+("00000"&i_o(	25	))
+("00000"&i_o(	26	))
+("00000"&i_o(	27	))
+("00000"&i_o(	28	))
+("00000"&i_o(	29	))
+("00000"&i_o(	30	))
+("00000"&i_o(	31	))
+("00000"&i_o(	32	))
+("00000"&i_o(	33	))
+("00000"&i_o(	34	))
+("00000"&i_o(	35	))
+("00000"&i_o(	36	))
+("00000"&i_o(	37	))
+("00000"&i_o(	38	))
+("00000"&i_o(	39	))
+("00000"&i_o(	40	))
+("00000"&i_o(	41	))
+("00000"&i_o(	42	))
+("00000"&i_o(	43	))
+("00000"&i_o(	44	))
+("00000"&i_o(	45	))
+("00000"&i_o(	46	))
+("00000"&i_o(	47	))
+("00000"&i_o(	48	))
+("00000"&i_o(	49	))
+("00000"&i_o(	50	));

ok <= '1' WHEN io_sum = "000001" ELSE '0';

LED3 <= X"F" when ok='1' else X"0";

sound1 <= timer(14) and ok and io_test_mode and INITB;

spkr <= sound1 when music_mode='0' else sound2;

end Behavioral;